"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_maintenancemode_maintenancemoderesponse import (
    V3ServicesMaintenanceModeMaintenanceModeResponse,
    V3ServicesMaintenanceModeMaintenanceModeResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class MaintenanceModeGetMaintenanceModeRequestTypedDict(TypedDict):
    service_id: str


class MaintenanceModeGetMaintenanceModeRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class MaintenanceModeGetMaintenanceModeResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3ServicesMaintenanceModeMaintenanceModeResponseTypedDict]


class MaintenanceModeGetMaintenanceModeResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V3ServicesMaintenanceModeMaintenanceModeResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_runbooks_runbookresponse import (
    V3RunbooksRunbookResponse,
    V3RunbooksRunbookResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class RunbooksGetRunbookByIDRequestTypedDict(TypedDict):
    runbook_id: str


class RunbooksGetRunbookByIDRequest(BaseModel):
    runbook_id: Annotated[
        str,
        pydantic.Field(alias="runbookID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class RunbooksGetRunbookByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3RunbooksRunbookResponseTypedDict


class RunbooksGetRunbookByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3RunbooksRunbookResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_overlay_renderdedupkeytemplaterequest import (
    V3ServicesOverlayRenderDedupKeyTemplateRequest,
    V3ServicesOverlayRenderDedupKeyTemplateRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class OverlayRenderDedupKeyTemplateRequestTypedDict(TypedDict):
    service_id: str
    v3_services_overlay_render_dedup_key_template_request: (
        V3ServicesOverlayRenderDedupKeyTemplateRequestTypedDict
    )


class OverlayRenderDedupKeyTemplateRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_services_overlay_render_dedup_key_template_request: Annotated[
        V3ServicesOverlayRenderDedupKeyTemplateRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class OverlayRenderDedupKeyTemplateResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: str


class OverlayRenderDedupKeyTemplateResponse(BaseModel):
    r"""The request has succeeded."""

    data: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3ServicesDependenciesCreateOrUpdateDependenciesRequestTypedDict(TypedDict):
    dependencies: List[str]


class V3ServicesDependenciesCreateOrUpdateDependenciesRequest(BaseModel):
    dependencies: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V3IncidentsResponseNoteTypedDict(TypedDict):
    r"""Represents a response note in an incident."""

    time_of_note: datetime
    note: str
    user_id: str


class V3IncidentsResponseNote(BaseModel):
    r"""Represents a response note in an incident."""

    time_of_note: Annotated[datetime, pydantic.Field(alias="timeOfNote")]

    note: str

    user_id: Annotated[str, pydantic.Field(alias="userId")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_incidentexportasyncresponse import (
    V3IncidentsIncidentExportAsyncResponse,
    V3IncidentsIncidentExportAsyncResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class IncidentsIncidentExportAsyncBodyTypedDict(TypedDict):
    r"""The body type of the operation request or response."""

    data: V3IncidentsIncidentExportAsyncResponseTypedDict


class IncidentsIncidentExportAsyncBody(BaseModel):
    r"""The body type of the operation request or response."""

    data: V3IncidentsIncidentExportAsyncResponse


class IncidentsIncidentExportAsyncResponseBodyTypedDict(TypedDict):
    r"""The request has been accepted for processing, but processing has not yet completed."""

    body: IncidentsIncidentExportAsyncBodyTypedDict
    r"""The body type of the operation request or response."""


class IncidentsIncidentExportAsyncResponseBody(BaseModel):
    r"""The request has been accepted for processing, but processing has not yet completed."""

    body: IncidentsIncidentExportAsyncBody
    r"""The body type of the operation request or response."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_componentgroups_getcomponentgroupbyidresponse import (
    V4StatusPagesComponentGroupsGetComponentGroupByIDResponse,
    V4StatusPagesComponentGroupsGetComponentGroupByIDResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class ComponentGroupsGetComponentGroupByIDRequestTypedDict(TypedDict):
    statuspage_id: str
    group_id: str


class ComponentGroupsGetComponentGroupByIDRequest(BaseModel):
    statuspage_id: Annotated[
        str,
        pydantic.Field(alias="statuspageID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    group_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class ComponentGroupsGetComponentGroupByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4StatusPagesComponentGroupsGetComponentGroupByIDResponseTypedDict


class ComponentGroupsGetComponentGroupByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4StatusPagesComponentGroupsGetComponentGroupByIDResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import date
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FiltersTypedDict(TypedDict):
    start_date: date
    end_date: date
    resource: NotRequired[List[str]]
    action: NotRequired[List[str]]
    actor: NotRequired[List[str]]
    team: NotRequired[List[str]]
    client: NotRequired[List[str]]


class Filters(BaseModel):
    start_date: Annotated[date, pydantic.Field(alias="startDate")]

    end_date: Annotated[date, pydantic.Field(alias="endDate")]

    resource: Optional[List[str]] = None

    action: Optional[List[str]] = None

    actor: Optional[List[str]] = None

    team: Optional[List[str]] = None

    client: Optional[List[str]] = None


ExportType = Literal[
    "csv",
    "json",
]


class V3AuditLogsExportAuditLogsRequestTypedDict(TypedDict):
    r"""Request model for exporting audit logs"""

    filters: FiltersTypedDict
    name: str
    export_type: ExportType
    description: NotRequired[str]


class V3AuditLogsExportAuditLogsRequest(BaseModel):
    r"""Request model for exporting audit logs"""

    filters: Filters

    name: str

    export_type: Annotated[ExportType, pydantic.Field(alias="exportType")]

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_listworkflowapiresponse import (
    V3WorkflowsListWorkflowAPIResponse,
    V3WorkflowsListWorkflowAPIResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WorkflowsListWorkflowsRequestTypedDict(TypedDict):
    owner_id: str
    page_size: NotRequired[int]
    page_number: NotRequired[int]
    search: NotRequired[str]
    event: NotRequired[List[str]]
    actions: NotRequired[List[str]]
    tags: NotRequired[List[str]]
    owner: NotRequired[List[str]]
    created_by: NotRequired[List[str]]
    updated_by: NotRequired[List[str]]
    enabled: NotRequired[str]


class WorkflowsListWorkflowsRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    page_number: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    search: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    event: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    actions: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    tags: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    owner: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    created_by: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    updated_by: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    enabled: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None


class WorkflowsListWorkflowsResponseTypedDict(TypedDict):
    result: V3WorkflowsListWorkflowAPIResponseTypedDict


class WorkflowsListWorkflowsResponse(BaseModel):
    next: Callable[[], Optional[WorkflowsListWorkflowsResponse]]

    result: V3WorkflowsListWorkflowAPIResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class ExportDeleteIcalLinkRequestTypedDict(TypedDict):
    schedule_id: str
    my_on_call: bool


class ExportDeleteIcalLinkRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    my_on_call: Annotated[
        bool,
        pydantic.Field(alias="myOnCall"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]


class ExportDeleteIcalLinkResponseBodyTypedDict(TypedDict):
    r"""There is no content to send for this request, but the headers may be useful."""


class ExportDeleteIcalLinkResponseBody(BaseModel):
    r"""There is no content to send for this request, but the headers may be useful."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .common_v3_entityowner import CommonV3EntityOwner, CommonV3EntityOwnerTypedDict
from .v3_runbooks_step import V3RunbooksStep, V3RunbooksStepTypedDict
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3RunbooksCreateRunbookRequestTypedDict(TypedDict):
    r"""Represents the request body for creating a new runbook."""

    name: str
    r"""The name of the runbook."""
    steps: List[V3RunbooksStepTypedDict]
    r"""The steps that make up the runbook."""
    owner_id: str
    r"""The ID of the team that owns this runbook."""
    entity_owner: NotRequired[CommonV3EntityOwnerTypedDict]
    r"""The owner of the entity."""


class V3RunbooksCreateRunbookRequest(BaseModel):
    r"""Represents the request body for creating a new runbook."""

    name: str
    r"""The name of the runbook."""

    steps: List[V3RunbooksStep]
    r"""The steps that make up the runbook."""

    owner_id: str
    r"""The ID of the team that owns this runbook."""

    entity_owner: Optional[CommonV3EntityOwner] = None
    r"""The owner of the entity."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_incidenteventresponse import (
    V3IncidentsIncidentEventResponse,
    V3IncidentsIncidentEventResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IncidentsGetIncidentEventsRequestTypedDict(TypedDict):
    incident_id: str
    offset: str
    r"""non zero value"""
    limit: str
    r"""non zero value, maximum is 10"""
    sort: str
    r"""sort it by either asc or desc"""
    deduped: str
    r"""if set to true, it will return only the deduped events.

    if set to false, it will return only the non-deduped event.

    otherwise it will return all the events
    """


class IncidentsGetIncidentEventsRequest(BaseModel):
    incident_id: Annotated[
        str,
        pydantic.Field(alias="incidentID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    offset: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""non zero value"""

    limit: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""non zero value, maximum is 10"""

    sort: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""sort it by either asc or desc"""

    deduped: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""if set to true, it will return only the deduped events.

    if set to false, it will return only the non-deduped event.

    otherwise it will return all the events
    """


class IncidentsGetIncidentEventsDataTypedDict(TypedDict):
    events: List[V3IncidentsIncidentEventResponseTypedDict]


class IncidentsGetIncidentEventsData(BaseModel):
    events: List[V3IncidentsIncidentEventResponse]


class IncidentsGetIncidentEventsMetaTypedDict(TypedDict):
    total: int
    count: int
    current: str
    next: NotRequired[str]


class IncidentsGetIncidentEventsMeta(BaseModel):
    total: int

    count: int

    current: str

    next: Optional[str] = None


class IncidentsGetIncidentEventsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: IncidentsGetIncidentEventsDataTypedDict
    meta: IncidentsGetIncidentEventsMetaTypedDict


class IncidentsGetIncidentEventsResponse(BaseModel):
    r"""The request has succeeded."""

    data: IncidentsGetIncidentEventsData

    meta: IncidentsGetIncidentEventsMeta



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3WorkflowsIssueStatusAndMessageTypedDict(TypedDict):
    messages: List[str]
    status_id: int


class V3WorkflowsIssueStatusAndMessage(BaseModel):
    messages: List[str]

    status_id: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3IncidentsRunbooksAttachRunbooksRequestTypedDict(TypedDict):
    runbooks: List[str]


class V3IncidentsRunbooksAttachRunbooksRequest(BaseModel):
    runbooks: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V3IncidentsAssignedToTypedDict(TypedDict):
    r"""Schema for an assigned entity in an incident."""

    id: str
    type: str
    time_of_assignment: datetime
    reason: str


class V3IncidentsAssignedTo(BaseModel):
    r"""Schema for an assigned entity in an incident."""

    id: str

    type: str

    time_of_assignment: Annotated[datetime, pydantic.Field(alias="timeOfAssignment")]

    reason: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_entityownerupdate import (
    V3GlobalEventRulesEntityOwnerUpdate,
    V3GlobalEventRulesEntityOwnerUpdateTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3GlobalEventRulesUpdateGlobalEventRuleRequestTypedDict(TypedDict):
    name: NotRequired[str]
    description: NotRequired[str]
    entity_owner: NotRequired[V3GlobalEventRulesEntityOwnerUpdateTypedDict]


class V3GlobalEventRulesUpdateGlobalEventRuleRequest(BaseModel):
    name: Optional[str] = None

    description: Optional[str] = None

    entity_owner: Optional[V3GlobalEventRulesEntityOwnerUpdate] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V4StatusPagesStatusPageStatusesResponseTypedDict(TypedDict):
    id: int
    name: str
    status: str
    description: str
    color: str


class V4StatusPagesStatusPageStatusesResponse(BaseModel):
    id: int

    name: str

    status: str

    description: str

    color: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_slo_sloaction import V3SLOSLOAction, V3SLOSLOActionTypedDict
from .v3_slo_slomonitoringcheck import (
    V3SLOSLOMonitoringCheck,
    V3SLOSLOMonitoringCheckTypedDict,
)
from .v3_slo_sloownertype import V3SLOSLOOwnerType
from .v3_slo_timeintervaltype import V3SLOTimeIntervalType
from datetime import datetime
from pydantic import model_serializer
from squadcast_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3SLOSLOTagsTypedDict(TypedDict):
    pass


class V3SLOSLOTags(BaseModel):
    pass


class V3SloSloTypedDict(TypedDict):
    id: int
    name: str
    time_interval_type: V3SLOTimeIntervalType
    service_ids: List[str]
    slis: List[str]
    target_slo: float
    start_time: datetime
    end_time: datetime
    allocated_error_budget: float
    is_active: bool
    owner_type: str
    owner_id: str
    org_id: str
    description: NotRequired[str]
    current_slo: NotRequired[float]
    is_healthy: NotRequired[bool]
    remaining_error_budget: NotRequired[float]
    tags: NotRequired[Nullable[V3SLOSLOTagsTypedDict]]
    incident_count: NotRequired[int]
    false_positive_count: NotRequired[int]
    slo_monitoring_checks: NotRequired[List[V3SLOSLOMonitoringCheckTypedDict]]
    slo_actions: NotRequired[Nullable[List[V3SLOSLOActionTypedDict]]]
    slo_owner_id: NotRequired[str]
    slo_owner_type: NotRequired[V3SLOSLOOwnerType]
    created_at: NotRequired[datetime]
    updated_at: NotRequired[datetime]
    deleted_at: NotRequired[Nullable[datetime]]
    duration_in_days: NotRequired[int]


class V3SloSlo(BaseModel):
    id: int

    name: str

    time_interval_type: V3SLOTimeIntervalType

    service_ids: List[str]

    slis: List[str]

    target_slo: float

    start_time: datetime

    end_time: datetime

    allocated_error_budget: float

    is_active: bool

    owner_type: str

    owner_id: str

    org_id: str

    description: Optional[str] = None

    current_slo: Optional[float] = None

    is_healthy: Optional[bool] = None

    remaining_error_budget: Optional[float] = None

    tags: OptionalNullable[V3SLOSLOTags] = UNSET

    incident_count: Optional[int] = None

    false_positive_count: Optional[int] = None

    slo_monitoring_checks: Optional[List[V3SLOSLOMonitoringCheck]] = None

    slo_actions: OptionalNullable[List[V3SLOSLOAction]] = UNSET

    slo_owner_id: Optional[str] = None

    slo_owner_type: Optional[V3SLOSLOOwnerType] = None

    created_at: Optional[datetime] = None

    updated_at: Optional[datetime] = None

    deleted_at: OptionalNullable[datetime] = UNSET

    duration_in_days: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "current_slo",
            "is_healthy",
            "remaining_error_budget",
            "tags",
            "incident_count",
            "false_positive_count",
            "slo_monitoring_checks",
            "slo_actions",
            "slo_owner_id",
            "slo_owner_type",
            "created_at",
            "updated_at",
            "deleted_at",
            "duration_in_days",
        ]
        nullable_fields = ["tags", "slo_actions", "deleted_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class MessageOverlayTypedDict(TypedDict):
    template: str


class MessageOverlay(BaseModel):
    template: str


class DescriptionOverlayTypedDict(TypedDict):
    template: str


class DescriptionOverlay(BaseModel):
    template: str


class V3ServicesOverlayUpdateCustomContentTemplateOverlayRequestTypedDict(TypedDict):
    overlay_template_type: str
    message_overlay: MessageOverlayTypedDict
    description_overlay: DescriptionOverlayTypedDict


class V3ServicesOverlayUpdateCustomContentTemplateOverlayRequest(BaseModel):
    overlay_template_type: str

    message_overlay: MessageOverlay

    description_overlay: DescriptionOverlay



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3IncidentsBulkIncidentIDsRequestTypedDict(TypedDict):
    r"""Request body for bulk incident operations (acknowledge, resolve)."""

    incident_ids: List[str]


class V3IncidentsBulkIncidentIDsRequest(BaseModel):
    r"""Request body for bulk incident operations (acknowledge, resolve)."""

    incident_ids: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3ServicesSuppressionRulesCustomRepetitionTypedDict(TypedDict):
    repeats_count: int
    repeats: str
    repeats_on_weekdays: List[int]
    repeats_on_month: str


class V3ServicesSuppressionRulesCustomRepetition(BaseModel):
    repeats_count: int

    repeats: str

    repeats_on_weekdays: List[int]

    repeats_on_month: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class UsersDeleteUserRequestTypedDict(TypedDict):
    user_id: str
    swap_user_id: str
    suppress_incidents: bool
    reassign_incidents: bool


class UsersDeleteUserRequest(BaseModel):
    user_id: str

    swap_user_id: str

    suppress_incidents: bool

    reassign_incidents: bool


class UsersDeleteUserDataTypedDict(TypedDict):
    pass


class UsersDeleteUserData(BaseModel):
    pass


class UsersDeleteUserResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: UsersDeleteUserDataTypedDict


class UsersDeleteUserResponse(BaseModel):
    r"""The request has succeeded."""

    data: UsersDeleteUserData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V4StatusPagesIssuesIssueStateMessageTypedDict(TypedDict):
    id: NotRequired[int]
    text: NotRequired[str]
    timestamp: NotRequired[datetime]


class V4StatusPagesIssuesIssueStateMessage(BaseModel):
    id: Optional[int] = None

    text: Optional[str] = None

    timestamp: Optional[datetime] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_auditlogs_auditlogsexporthistoryresponse import (
    V3AuditLogsAuditLogsExportHistoryResponse,
    V3AuditLogsAuditLogsExportHistoryResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3AuditLogsGetAuditLogExportHistoryByIDResponseTypedDict(TypedDict):
    r"""Response model for getting audit log export history by ID"""

    data: V3AuditLogsAuditLogsExportHistoryResponseTypedDict
    r"""Response model for audit logs export history"""


class V3AuditLogsGetAuditLogExportHistoryByIDResponse(BaseModel):
    r"""Response model for getting audit log export history by ID"""

    data: V3AuditLogsAuditLogsExportHistoryResponse
    r"""Response model for audit logs export history"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3IncidentsIncidentPriorityUpdateRequestTypedDict(TypedDict):
    r"""Request body for updating incident priority."""

    priority: NotRequired[str]


class V3IncidentsIncidentPriorityUpdateRequest(BaseModel):
    r"""Request body for updating incident priority."""

    priority: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_suppressionrules_basicexpression import (
    V3ServicesSuppressionRulesBasicExpression,
    V3ServicesSuppressionRulesBasicExpressionTypedDict,
)
from .v3_services_suppressionrules_timeslot import (
    V3ServicesSuppressionRulesTimeslot,
    V3ServicesSuppressionRulesTimeslotTypedDict,
)
from datetime import datetime
from pydantic import model_serializer
from squadcast_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from typing import List
from typing_extensions import TypedDict


class V3ServicesSuppressionRulesSuppressionRuleResponseTypedDict(TypedDict):
    rule_id: str
    created_at: datetime
    updated_at: datetime
    created_by: str
    updated_by: str
    description: str
    expression: str
    is_basic: bool
    is_timebased: bool
    timeslots: Nullable[List[V3ServicesSuppressionRulesTimeslotTypedDict]]
    basic_expression: List[V3ServicesSuppressionRulesBasicExpressionTypedDict]


class V3ServicesSuppressionRulesSuppressionRuleResponse(BaseModel):
    rule_id: str

    created_at: datetime

    updated_at: datetime

    created_by: str

    updated_by: str

    description: str

    expression: str

    is_basic: bool

    is_timebased: bool

    timeslots: Nullable[List[V3ServicesSuppressionRulesTimeslot]]

    basic_expression: List[V3ServicesSuppressionRulesBasicExpression]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["timeslots"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
import io
from jsonpath import JSONPath
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, IO, List, Mapping, Optional, Union


class EscalationPoliciesSDK(BaseSDK):
    def get_by_team(
        self,
        *,
        owner_id: str,
        page_number: Optional[int] = None,
        page_size: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.EscalationPoliciesGetEscalationPolicyByTeamResponse]:
        r"""Get Escalation Policy By team

        Returns all escalation policy details of the given `ownerID` (teamId) in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param owner_id: here owner_id represents team_id, if  team_id is not provided, it will return escalation policies of all teams.
        :param page_number:
        :param page_size:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesGetEscalationPolicyByTeamRequest(
            owner_id=owner_id,
            page_number=page_number,
            page_size=page_size,
        )

        req = self._build_request(
            method="GET",
            path="/v3/escalation-policies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_getEscalationPolicyByTeam",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> (
            Optional[models.EscalationPoliciesGetEscalationPolicyByTeamResponse]
        ):
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number if not request.page_number is None else 1
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size if not request.page_size is None else 0
            if len(results[0]) < limit:
                return None

            return self.get_by_team(
                owner_id=owner_id,
                page_number=next_page,
                page_size=page_size,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.EscalationPoliciesGetEscalationPolicyByTeamResponse(
                result=unmarshal_json_response(
                    models.EscalationPoliciesGetEscalationPolicyByTeamResponseBody,
                    http_res,
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_by_team_async(
        self,
        *,
        owner_id: str,
        page_number: Optional[int] = None,
        page_size: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.EscalationPoliciesGetEscalationPolicyByTeamResponse]:
        r"""Get Escalation Policy By team

        Returns all escalation policy details of the given `ownerID` (teamId) in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param owner_id: here owner_id represents team_id, if  team_id is not provided, it will return escalation policies of all teams.
        :param page_number:
        :param page_size:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesGetEscalationPolicyByTeamRequest(
            owner_id=owner_id,
            page_number=page_number,
            page_size=page_size,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/escalation-policies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_getEscalationPolicyByTeam",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> (
            Optional[models.EscalationPoliciesGetEscalationPolicyByTeamResponse]
        ):
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number if not request.page_number is None else 1
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size if not request.page_size is None else 0
            if len(results[0]) < limit:
                return None

            return self.get_by_team(
                owner_id=owner_id,
                page_number=next_page,
                page_size=page_size,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.EscalationPoliciesGetEscalationPolicyByTeamResponse(
                result=unmarshal_json_response(
                    models.EscalationPoliciesGetEscalationPolicyByTeamResponseBody,
                    http_res,
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        owner_id: str,
        name: str,
        description: str,
        repetition: int,
        repeat_after: int,
        rules: Union[
            List[models.V3EscalationPoliciesEscalationPolicyRule],
            List[models.V3EscalationPoliciesEscalationPolicyRuleTypedDict],
        ],
        enable_incident_reminders: bool,
        incident_reminder_rules: Union[
            List[models.V3EscalationPoliciesIncidentReminderRule],
            List[models.V3EscalationPoliciesIncidentReminderRuleTypedDict],
        ],
        enable_incident_retrigger: bool,
        retrigger_after: int,
        entity_owner: Optional[
            Union[models.CommonV3EntityOwner, models.CommonV3EntityOwnerTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3EscalationPoliciesEscalationPolicyResponse:
        r"""Create Escalation Policies

        Add escalation policy to the organization. Returns the escalation policy object in response.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param owner_id: The ID of the team that owns this escalation policy.
        :param name: The name of the escalation policy.
        :param description: A description of the escalation policy.
        :param repetition: The number of times the entire policy should be repeated.
        :param repeat_after: The time in minutes after which the policy should be repeated.
        :param rules: The rules that define the escalation steps.
        :param enable_incident_reminders: Enable or disable incident reminders.
        :param incident_reminder_rules: The rules for incident reminders.
        :param enable_incident_retrigger: Enable or disable automatic incident re-triggering.
        :param retrigger_after: The time in hours after which an incident should be re-triggered.
        :param entity_owner: The owner of the entity.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3EscalationPoliciesCreateEscalationPolicyRequest(
            owner_id=owner_id,
            name=name,
            description=description,
            repetition=repetition,
            repeat_after=repeat_after,
            rules=utils.get_pydantic_model(
                rules, List[models.V3EscalationPoliciesEscalationPolicyRule]
            ),
            enable_incident_reminders=enable_incident_reminders,
            incident_reminder_rules=utils.get_pydantic_model(
                incident_reminder_rules,
                List[models.V3EscalationPoliciesIncidentReminderRule],
            ),
            enable_incident_retrigger=enable_incident_retrigger,
            retrigger_after=retrigger_after,
            entity_owner=utils.get_pydantic_model(
                entity_owner, Optional[models.CommonV3EntityOwner]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v3/escalation-policies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.V3EscalationPoliciesCreateEscalationPolicyRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_createEscalationPolicies",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.EscalationPoliciesCreateEscalationPoliciesResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        owner_id: str,
        name: str,
        description: str,
        repetition: int,
        repeat_after: int,
        rules: Union[
            List[models.V3EscalationPoliciesEscalationPolicyRule],
            List[models.V3EscalationPoliciesEscalationPolicyRuleTypedDict],
        ],
        enable_incident_reminders: bool,
        incident_reminder_rules: Union[
            List[models.V3EscalationPoliciesIncidentReminderRule],
            List[models.V3EscalationPoliciesIncidentReminderRuleTypedDict],
        ],
        enable_incident_retrigger: bool,
        retrigger_after: int,
        entity_owner: Optional[
            Union[models.CommonV3EntityOwner, models.CommonV3EntityOwnerTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3EscalationPoliciesEscalationPolicyResponse:
        r"""Create Escalation Policies

        Add escalation policy to the organization. Returns the escalation policy object in response.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param owner_id: The ID of the team that owns this escalation policy.
        :param name: The name of the escalation policy.
        :param description: A description of the escalation policy.
        :param repetition: The number of times the entire policy should be repeated.
        :param repeat_after: The time in minutes after which the policy should be repeated.
        :param rules: The rules that define the escalation steps.
        :param enable_incident_reminders: Enable or disable incident reminders.
        :param incident_reminder_rules: The rules for incident reminders.
        :param enable_incident_retrigger: Enable or disable automatic incident re-triggering.
        :param retrigger_after: The time in hours after which an incident should be re-triggered.
        :param entity_owner: The owner of the entity.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3EscalationPoliciesCreateEscalationPolicyRequest(
            owner_id=owner_id,
            name=name,
            description=description,
            repetition=repetition,
            repeat_after=repeat_after,
            rules=utils.get_pydantic_model(
                rules, List[models.V3EscalationPoliciesEscalationPolicyRule]
            ),
            enable_incident_reminders=enable_incident_reminders,
            incident_reminder_rules=utils.get_pydantic_model(
                incident_reminder_rules,
                List[models.V3EscalationPoliciesIncidentReminderRule],
            ),
            enable_incident_retrigger=enable_incident_retrigger,
            retrigger_after=retrigger_after,
            entity_owner=utils.get_pydantic_model(
                entity_owner, Optional[models.CommonV3EntityOwner]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/escalation-policies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.V3EscalationPoliciesCreateEscalationPolicyRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_createEscalationPolicies",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.EscalationPoliciesCreateEscalationPoliciesResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def remove(
        self,
        *,
        escalation_policy_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Remove Escalation Policy

        Remove escalation policy from the organization. Upon success, the escalation policy will be removed from the organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param escalation_policy_id: (Required) escalation policy ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesRemoveEscalationPolicyRequest(
            escalation_policy_id=escalation_policy_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v3/escalation-policies/{escalationPolicyID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_removeEscalationPolicy",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = utils.stream_to_bytes(http_res)
            return http_res_bytes
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def remove_async(
        self,
        *,
        escalation_policy_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Remove Escalation Policy

        Remove escalation policy from the organization. Upon success, the escalation policy will be removed from the organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param escalation_policy_id: (Required) escalation policy ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesRemoveEscalationPolicyRequest(
            escalation_policy_id=escalation_policy_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v3/escalation-policies/{escalationPolicyID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_removeEscalationPolicy",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = await utils.stream_to_bytes_async(http_res)
            return http_res_bytes
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        escalation_policy_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3EscalationPoliciesEscalationPolicyResponse:
        r"""Get Escalation Policy By ID

        Returns an escalation policy details of the given `escalationPolicyID` in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param escalation_policy_id: (Required) escalation policy ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesGetEscalationPolicyByIDRequest(
            escalation_policy_id=escalation_policy_id,
        )

        req = self._build_request(
            method="GET",
            path="/v3/escalation-policies/{escalationPolicyID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_getEscalationPolicyById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EscalationPoliciesGetEscalationPolicyByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        escalation_policy_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3EscalationPoliciesEscalationPolicyResponse:
        r"""Get Escalation Policy By ID

        Returns an escalation policy details of the given `escalationPolicyID` in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param escalation_policy_id: (Required) escalation policy ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesGetEscalationPolicyByIDRequest(
            escalation_policy_id=escalation_policy_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/escalation-policies/{escalationPolicyID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_getEscalationPolicyById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EscalationPoliciesGetEscalationPolicyByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update(
        self,
        *,
        escalation_policy_id: str,
        v3_escalation_policies_update_escalation_policy_request: Union[
            bytes, IO[bytes], io.BufferedReader
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3EscalationPoliciesEscalationPolicyResponse:
        r"""Update Escalation Policy

        Update organization escalation policy details.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param escalation_policy_id: (Required) escalation policy ID
        :param v3_escalation_policies_update_escalation_policy_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesUpdateEscalationPolicyRequest(
            escalation_policy_id=escalation_policy_id,
            v3_escalation_policies_update_escalation_policy_request=v3_escalation_policies_update_escalation_policy_request,
        )

        req = self._build_request(
            method="POST",
            path="/v3/escalation-policies/{escalationPolicyID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_escalation_policies_update_escalation_policy_request,
                False,
                False,
                "raw",
                Union[bytes, IO[bytes], io.BufferedReader],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_updateEscalationPolicy",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EscalationPoliciesUpdateEscalationPolicyResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        escalation_policy_id: str,
        v3_escalation_policies_update_escalation_policy_request: Union[
            bytes, IO[bytes], io.BufferedReader
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3EscalationPoliciesEscalationPolicyResponse:
        r"""Update Escalation Policy

        Update organization escalation policy details.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param escalation_policy_id: (Required) escalation policy ID
        :param v3_escalation_policies_update_escalation_policy_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EscalationPoliciesUpdateEscalationPolicyRequest(
            escalation_policy_id=escalation_policy_id,
            v3_escalation_policies_update_escalation_policy_request=v3_escalation_policies_update_escalation_policy_request,
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/escalation-policies/{escalationPolicyID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_escalation_policies_update_escalation_policy_request,
                False,
                False,
                "raw",
                Union[bytes, IO[bytes], io.BufferedReader],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EscalationPolicies_updateEscalationPolicy",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EscalationPoliciesUpdateEscalationPolicyResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V4SquadsSquadMemberTypedDict(TypedDict):
    user_id: str
    role: NotRequired[str]
    r"""this field is required if you are using OBAC permission model"""


class V4SquadsSquadMember(BaseModel):
    user_id: str

    role: Optional[str] = None
    r"""this field is required if you are using OBAC permission model"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Any
from typing_extensions import TypedDict


class V3IncidentsCommunicationCardsCommunicationCardResponseTypedDict(TypedDict):
    r"""Represents a communication card."""

    created_at: str
    updated_at: str
    deleted_at: Any
    id: int
    incident_id: str
    url: str
    type: str
    title: str
    channel_id: str
    archived_at: Any


class V3IncidentsCommunicationCardsCommunicationCardResponse(BaseModel):
    r"""Represents a communication card."""

    created_at: str

    updated_at: str

    deleted_at: Any

    id: int

    incident_id: str

    url: str

    type: str

    title: str

    channel_id: str

    archived_at: Any



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_suppressionrules_createorupdatesuppressionrulesrequest import (
    V3ServicesSuppressionRulesCreateOrUpdateSuppressionRulesRequest,
    V3ServicesSuppressionRulesCreateOrUpdateSuppressionRulesRequestTypedDict,
)
from .v3_services_suppressionrules_suppressionrulescontainerresponse import (
    V3ServicesSuppressionRulesSuppressionRulesContainerResponse,
    V3ServicesSuppressionRulesSuppressionRulesContainerResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class SuppressionRulesCreateOrUpdateSuppressionRulesRequestTypedDict(TypedDict):
    service_id: str
    v3_services_suppression_rules_create_or_update_suppression_rules_request: (
        V3ServicesSuppressionRulesCreateOrUpdateSuppressionRulesRequestTypedDict
    )


class SuppressionRulesCreateOrUpdateSuppressionRulesRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_services_suppression_rules_create_or_update_suppression_rules_request: Annotated[
        V3ServicesSuppressionRulesCreateOrUpdateSuppressionRulesRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class SuppressionRulesCreateOrUpdateSuppressionRulesResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3ServicesSuppressionRulesSuppressionRulesContainerResponseTypedDict


class SuppressionRulesCreateOrUpdateSuppressionRulesResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3ServicesSuppressionRulesSuppressionRulesContainerResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .common_v3_entityowner import CommonV3EntityOwner, CommonV3EntityOwnerTypedDict
from .v3_escalationpolicies_escalationpolicyrule import (
    V3EscalationPoliciesEscalationPolicyRule,
    V3EscalationPoliciesEscalationPolicyRuleTypedDict,
)
from .v3_escalationpolicies_incidentreminderrule import (
    V3EscalationPoliciesIncidentReminderRule,
    V3EscalationPoliciesIncidentReminderRuleTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3EscalationPoliciesCreateEscalationPolicyRequestTypedDict(TypedDict):
    owner_id: str
    r"""The ID of the team that owns this escalation policy."""
    name: str
    r"""The name of the escalation policy."""
    description: str
    r"""A description of the escalation policy."""
    repetition: int
    r"""The number of times the entire policy should be repeated."""
    repeat_after: int
    r"""The time in minutes after which the policy should be repeated."""
    rules: List[V3EscalationPoliciesEscalationPolicyRuleTypedDict]
    r"""The rules that define the escalation steps."""
    enable_incident_reminders: bool
    r"""Enable or disable incident reminders."""
    incident_reminder_rules: List[V3EscalationPoliciesIncidentReminderRuleTypedDict]
    r"""The rules for incident reminders."""
    enable_incident_retrigger: bool
    r"""Enable or disable automatic incident re-triggering."""
    retrigger_after: int
    r"""The time in hours after which an incident should be re-triggered."""
    entity_owner: NotRequired[CommonV3EntityOwnerTypedDict]
    r"""The owner of the entity."""


class V3EscalationPoliciesCreateEscalationPolicyRequest(BaseModel):
    owner_id: str
    r"""The ID of the team that owns this escalation policy."""

    name: str
    r"""The name of the escalation policy."""

    description: str
    r"""A description of the escalation policy."""

    repetition: int
    r"""The number of times the entire policy should be repeated."""

    repeat_after: int
    r"""The time in minutes after which the policy should be repeated."""

    rules: List[V3EscalationPoliciesEscalationPolicyRule]
    r"""The rules that define the escalation steps."""

    enable_incident_reminders: bool
    r"""Enable or disable incident reminders."""

    incident_reminder_rules: List[V3EscalationPoliciesIncidentReminderRule]
    r"""The rules for incident reminders."""

    enable_incident_retrigger: bool
    r"""Enable or disable automatic incident re-triggering."""

    retrigger_after: int
    r"""The time in hours after which an incident should be re-triggered."""

    entity_owner: Optional[CommonV3EntityOwner] = None
    r"""The owner of the entity."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


V3WorkflowsWorkflowTrigger = Literal[
    "incident_triggered",
    "incident_acknowledged",
    "incident_resolved",
    "incident_reassigned",
    "incident_tag_updated",
    "incident_priority_updated",
    "incident_note_added",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_postmortems_postmortemattachmentrequest import (
    V3IncidentsPostmortemsPostmortemAttachmentRequest,
    V3IncidentsPostmortemsPostmortemAttachmentRequestTypedDict,
)
from .v3_incidents_postmortems_postmortemfollowup import (
    V3IncidentsPostmortemsPostmortemFollowUp,
    V3IncidentsPostmortemsPostmortemFollowUpTypedDict,
)
from .v3_incidents_postmortems_postmortemstatus import (
    V3IncidentsPostmortemsPostmortemStatus,
)
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3IncidentsPostmortemsUpdatePostmortemRequestTypedDict(TypedDict):
    r"""Request body for updating an existing postmortem."""

    owner_id: NotRequired[str]
    postmortem: NotRequired[str]
    follow_ups: NotRequired[List[V3IncidentsPostmortemsPostmortemFollowUpTypedDict]]
    new_attachments: NotRequired[
        List[V3IncidentsPostmortemsPostmortemAttachmentRequestTypedDict]
    ]
    title: NotRequired[str]
    status: NotRequired[V3IncidentsPostmortemsPostmortemStatus]
    r"""Represents the status of a postmortem."""


class V3IncidentsPostmortemsUpdatePostmortemRequest(BaseModel):
    r"""Request body for updating an existing postmortem."""

    owner_id: Optional[str] = None

    postmortem: Optional[str] = None

    follow_ups: Optional[List[V3IncidentsPostmortemsPostmortemFollowUp]] = None

    new_attachments: Optional[
        List[V3IncidentsPostmortemsPostmortemAttachmentRequest]
    ] = None

    title: Optional[str] = None

    status: Optional[V3IncidentsPostmortemsPostmortemStatus] = None
    r"""Represents the status of a postmortem."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V3IncidentsTagsRelevantPeopleLogTypedDict(TypedDict):
    r"""Represents a relevant people log entry."""

    user_id: str
    action_taken: str
    action_taken_by: str
    action_time: datetime


class V3IncidentsTagsRelevantPeopleLog(BaseModel):
    r"""Represents a relevant people log entry."""

    user_id: Annotated[str, pydantic.Field(alias="userId")]

    action_taken: Annotated[str, pydantic.Field(alias="actionTaken")]

    action_taken_by: Annotated[str, pydantic.Field(alias="actionTakenBy")]

    action_time: Annotated[datetime, pydantic.Field(alias="actionTime")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_rulesetruleresponse import (
    V3GlobalEventRulesRulesetRuleResponse,
    V3GlobalEventRulesRulesetRuleResponseTypedDict,
)
from .v3_globaleventrules_updaterulerequest import (
    V3GlobalEventRulesUpdateRuleRequest,
    V3GlobalEventRulesUpdateRuleRequestTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class GlobalEventRulesUpdateRuleByIDRequestTypedDict(TypedDict):
    ger_id: int
    alert_source_version: str
    alert_source_shortname: str
    rule_id: str
    v3_global_event_rules_update_rule_request: (
        V3GlobalEventRulesUpdateRuleRequestTypedDict
    )


class GlobalEventRulesUpdateRuleByIDRequest(BaseModel):
    ger_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    alert_source_version: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    alert_source_shortname: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    rule_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    v3_global_event_rules_update_rule_request: Annotated[
        V3GlobalEventRulesUpdateRuleRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class GlobalEventRulesUpdateRuleByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesRulesetRuleResponseTypedDict


class GlobalEventRulesUpdateRuleByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesRulesetRuleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsSlackMessageChannelUpdateName = Literal["slack_message_channel",]


class V3WorkflowsSlackMessageChannelUpdateDataTypedDict(TypedDict):
    channel_id: NotRequired[str]
    channel_name: NotRequired[str]
    message: NotRequired[str]


class V3WorkflowsSlackMessageChannelUpdateData(BaseModel):
    channel_id: Optional[str] = None

    channel_name: Optional[str] = None

    message: Optional[str] = None


class V3WorkflowsSlackMessageChannelUpdateTypedDict(TypedDict):
    name: V3WorkflowsSlackMessageChannelUpdateName
    data: NotRequired[V3WorkflowsSlackMessageChannelUpdateDataTypedDict]


class V3WorkflowsSlackMessageChannelUpdate(BaseModel):
    name: V3WorkflowsSlackMessageChannelUpdateName

    data: Optional[V3WorkflowsSlackMessageChannelUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3TeamsCreateTeamRequestTypedDict(TypedDict):
    name: str
    member_ids: List[str]
    description: NotRequired[str]


class V3TeamsCreateTeamRequest(BaseModel):
    name: str

    member_ids: List[str]

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3UsersAPITokenResponseTypedDict(TypedDict):
    id: str
    organization_id: str
    user_id: str
    token: str
    last_refresh: str
    rate_limit: int
    deleted: bool


class V3UsersAPITokenResponse(BaseModel):
    id: str

    organization_id: str

    user_id: str

    token: str

    last_refresh: str

    rate_limit: int

    deleted: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_participantgroup import V4ParticipantGroup, V4ParticipantGroupTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V4GetRotationParticipantsResponseTypedDict(TypedDict):
    rotation_id: int
    participants: List[V4ParticipantGroupTypedDict]


class V4GetRotationParticipantsResponse(BaseModel):
    rotation_id: Annotated[int, pydantic.Field(alias="rotationID")]

    participants: List[V4ParticipantGroup]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.squads_members import SquadsMembers
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class SquadsSDK(BaseSDK):
    members: SquadsMembers

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.members = SquadsMembers(self.sdk_configuration, parent_ref=self.parent_ref)

    def list(
        self,
        *,
        owner_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V4SquadsSquadResponse]:
        r"""Get All Squads

        This endpoint is used to get all the squads details of your organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-read` scope.

        :param owner_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsGetAllSquadsRequest(
            owner_id=owner_id,
        )

        req = self._build_request(
            method="GET",
            path="/v4/squads",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_getAllSquads",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SquadsGetAllSquadsResponse, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        owner_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V4SquadsSquadResponse]:
        r"""Get All Squads

        This endpoint is used to get all the squads details of your organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-read` scope.

        :param owner_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsGetAllSquadsRequest(
            owner_id=owner_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/squads",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_getAllSquads",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SquadsGetAllSquadsResponse, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        squad_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsSquadResponse:
        r"""Get Squad By ID

        This endpoint is used to get the squads details by id.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-read` scope.

        :param squad_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsGetSquadByIDRequest(
            squad_id=squad_id,
        )

        req = self._build_request(
            method="GET",
            path="/v4/squads/{squadID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_getSquadById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SquadsGetSquadByIDResponse, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        squad_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsSquadResponse:
        r"""Get Squad By ID

        This endpoint is used to get the squads details by id.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-read` scope.

        :param squad_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsGetSquadByIDRequest(
            squad_id=squad_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/squads/{squadID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_getSquadById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SquadsGetSquadByIDResponse, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update_v4(
        self,
        *,
        squad_id: str,
        name: Optional[str] = None,
        members: Optional[
            Union[
                List[models.V4SquadsSquadMember],
                List[models.V4SquadsSquadMemberTypedDict],
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsSquadResponse:
        r"""Update Squad

        This endpoint is used to update squad.

        The role will be considered only if your organization is on the OBAC permission model; otherwise, the role field will be ignored, and only the member will be added to the squad.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-create` scope.

        :param squad_id:
        :param name:
        :param members:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsUpdateSquadRequest(
            squad_id=squad_id,
            v4_squads_update_squad_request=models.V4SquadsUpdateSquadRequest(
                name=name,
                members=utils.get_pydantic_model(
                    members, Optional[List[models.V4SquadsSquadMember]]
                ),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v4/squads/{squadID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_squads_update_squad_request,
                False,
                False,
                "json",
                models.V4SquadsUpdateSquadRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_updateSquad",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SquadsUpdateSquadResponse, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_v4_async(
        self,
        *,
        squad_id: str,
        name: Optional[str] = None,
        members: Optional[
            Union[
                List[models.V4SquadsSquadMember],
                List[models.V4SquadsSquadMemberTypedDict],
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsSquadResponse:
        r"""Update Squad

        This endpoint is used to update squad.

        The role will be considered only if your organization is on the OBAC permission model; otherwise, the role field will be ignored, and only the member will be added to the squad.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-create` scope.

        :param squad_id:
        :param name:
        :param members:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsUpdateSquadRequest(
            squad_id=squad_id,
            v4_squads_update_squad_request=models.V4SquadsUpdateSquadRequest(
                name=name,
                members=utils.get_pydantic_model(
                    members, Optional[List[models.V4SquadsSquadMember]]
                ),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v4/squads/{squadID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_squads_update_squad_request,
                False,
                False,
                "json",
                models.V4SquadsUpdateSquadRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_updateSquad",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SquadsUpdateSquadResponse, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def remove_member(
        self,
        *,
        squad_id: str,
        member_id: str,
        replace_with: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsRemoveSquadMemberResponse:
        r"""Remove Squad Member

        This endpoint is used to update squad member's role.

        If you're attempting to remove a member whose role is 'owner', and that 'owner' is the last squad owner, then the 'replaceWith' query parameter is required. Setting 'replaceWith' to 'member' will promote another member as the owner, and the specified member will be removed.

        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-create` scope.

        :param squad_id:
        :param member_id:
        :param replace_with:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsRemoveSquadMemberRequest(
            squad_id=squad_id,
            member_id=member_id,
            replace_with=replace_with,
        )

        req = self._build_request(
            method="DELETE",
            path="/v4/squads/{squadID}/members/{memberID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_removeSquadMember",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V4SquadsRemoveSquadMemberResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def remove_member_async(
        self,
        *,
        squad_id: str,
        member_id: str,
        replace_with: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsRemoveSquadMemberResponse:
        r"""Remove Squad Member

        This endpoint is used to update squad member's role.

        If you're attempting to remove a member whose role is 'owner', and that 'owner' is the last squad owner, then the 'replaceWith' query parameter is required. Setting 'replaceWith' to 'member' will promote another member as the owner, and the specified member will be removed.

        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-create` scope.

        :param squad_id:
        :param member_id:
        :param replace_with:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsRemoveSquadMemberRequest(
            squad_id=squad_id,
            member_id=member_id,
            replace_with=replace_with,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v4/squads/{squadID}/members/{memberID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_removeSquadMember",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V4SquadsRemoveSquadMemberResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def squads_delete_squad(
        self,
        *,
        squad_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Delete Squad

        This endpoint is used to delete the squad. Squad should not be assigned to any incident or part of any escalation policy.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-update` scope.

        :param squad_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsDeleteSquadRequest(
            squad_id=squad_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v4/squads/{squadID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_deleteSquad",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = utils.stream_to_bytes(http_res)
            return http_res_bytes
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def squads_delete_squad_async(
        self,
        *,
        squad_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Delete Squad

        This endpoint is used to delete the squad. Squad should not be assigned to any incident or part of any escalation policy.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-update` scope.

        :param squad_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsDeleteSquadRequest(
            squad_id=squad_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v4/squads/{squadID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_deleteSquad",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = await utils.stream_to_bytes_async(http_res)
            return http_res_bytes
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsIncidentActionsCircleCIErrorMetaTypedDict(TypedDict):
    r"""Represents the metadata for a CircleCI error response."""

    status_code: int
    error_message: str


class V3IncidentsIncidentActionsCircleCIErrorMeta(BaseModel):
    r"""Represents the metadata for a CircleCI error response."""

    status_code: int

    error_message: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_ruleaction import (
    V3GlobalEventRulesRuleAction,
    V3GlobalEventRulesRuleActionTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3GlobalEventRulesCreateRulesetRequestTypedDict(TypedDict):
    alert_source_shortname: str
    alert_source_version: str
    catch_all_action: NotRequired[V3GlobalEventRulesRuleActionTypedDict]


class V3GlobalEventRulesCreateRulesetRequest(BaseModel):
    alert_source_shortname: str

    alert_source_version: str

    catch_all_action: Optional[V3GlobalEventRulesRuleAction] = None
